@@ -12,7 +12,6 @@
 from datetime import datetime, date, time
 
 from soxspipe.commonutils.filenamer import filenamer
-from fundamentals.renderer import list_of_dictionaries
 import unicodecsv as csv
 import collections
 from soxspipe.commonutils.toolkit import unpack_order_table
@@ -133,6 +132,7 @@ class detect_order_edges(_base_detect):
         self.orderDeg = self.recipeSettings["order-deg"]
 
         self.inst = flatFrame.header[kw("INSTRUME")]
+        # SET IMAGE ORIENTATION
         if self.inst == "SOXS":
             self.axisA = "y"
             self.axisB = "x"
@@ -175,7 +175,7 @@ class detect_order_edges(_base_detect):
 
         # UNPACK THE ORDER TABLE (CENTRE LOCATION ONLY AT THIS STAGE)
         orderPolyTable, orderPixelTable, orderMetaTable = unpack_order_table(
-            log=self.log, orderTablePath=self.orderCentreTable, binx=self.binx, biny=self.biny)
+            log=self.log, orderTablePath=self.orderCentreTable, binx=self.binx, biny=self.biny, pixelDelta=9)
 
         # ADD MIN AND MAX FLUX THRESHOLDS TO ORDER TABLE
         print("\tDETERMINING ORDER FLUX THRESHOLDS")
@@ -225,7 +225,7 @@ class detect_order_edges(_base_detect):
             orderPixelTable[f"order_pow_{i}"] = orderPixelTable["order"].pow(i)
 
         # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
-        upperCoeff, orderPixelTable = self.fit_global_polynomial(
+        upperCoeff, orderPixelTable, clippedUpper = self.fit_global_polynomial(
             pixelList=orderPixelTable,
             axisBCol=f"{self.axisB}coord",
             axisACol=f"{self.axisA}coord_upper",
@@ -236,10 +236,12 @@ class detect_order_edges(_base_detect):
         # RENAME SOME INDIVIDUALLY
         orderPixelTable.rename(columns={
             f"{self.axisA}_fit": f"{self.axisA}coord_upper_fit", f"{self.axisA}_fit_res": f"{self.axisA}coord_upper_fit_res"}, inplace=True)
+        clippedUpper.rename(columns={
+            f"{self.axisA}_fit": f"{self.axisA}coord_upper_fit", f"{self.axisA}_fit_res": f"{self.axisA}coord_upper_fit_res"}, inplace=True)
 
         # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
         print("\tFITTING POLYNOMIALS TO MEASURED PIXEL-POSITIONS AT LOWER ORDER-EDGES\n")
-        lowerCoeff, orderPixelTable = self.fit_global_polynomial(
+        lowerCoeff, orderPixelTable, clippedLower = self.fit_global_polynomial(
             pixelList=orderPixelTable,
             axisBCol=f"{self.axisB}coord",
             axisACol=f"{self.axisA}coord_lower",
@@ -250,6 +252,8 @@ class detect_order_edges(_base_detect):
         # RENAME SOME INDIVIDUALLY
         orderPixelTable.rename(columns={
             f"{self.axisA}_fit": f"{self.axisA}coord_lower_fit", f"{self.axisA}_fit_res": f"{self.axisA}coord_lower_fit_res"}, inplace=True)
+        clippedLower.rename(columns={
+            f"{self.axisA}_fit": f"{self.axisA}coord_lower_fit", f"{self.axisA}_fit_res": f"{self.axisA}coord_lower_fit_res"}, inplace=True)
 
         # orderLocations[o] = coeff
         coeff_dict = {
@@ -265,18 +269,11 @@ class detect_order_edges(_base_detect):
                 coeff_dict[f'edgelow_c{i}{j}'] = lowerCoeff[n_coeff]
                 n_coeff += 1
         coeffColumns = coeff_dict.keys()
-        dataSet = list_of_dictionaries(
-            log=self.log,
-            listOfDictionaries=[coeff_dict]
-        )
 
         # RETURN BREAKDOWN OF ORDER EDGE DETECTION POSITION COUNTS (NEEDED TO COMPARE D and Q LAMPS)
         orderDetectionCounts = orderPixelTable['order'].value_counts(normalize=False).sort_index().to_frame()
 
-        # WRITE CSV DATA TO PANDAS DATAFRAME TO ASTROPY TABLE TO FITS
-        fakeFile = StringIO(dataSet.csv())
-        orderEdgePolyTable = pd.read_csv(fakeFile, index_col=False, na_values=['NA', 'MISSING'])
-        fakeFile.close()
+        orderEdgePolyTable = pd.DataFrame([coeff_dict])
 
         # MERGE DATAFRAMES
         cols_to_use = orderEdgePolyTable.columns.difference(orderPolyTable.columns)
@@ -290,7 +287,9 @@ class detect_order_edges(_base_detect):
         plotPath = self.plot_results(
             orderPixelTable=orderPixelTable,
             orderPolyTable=orderPolyTable,
-            orderMetaTable=orderMetaTable
+            orderMetaTable=orderMetaTable,
+            clippedDataUpper=clippedUpper,
+            clippedDataLower=clippedLower
         )
 
         # WRITE OUT THE FITS TO THE ORDER CENTRE TABLE
@@ -337,7 +336,7 @@ class detect_order_edges(_base_detect):
                 "soxspipe_recipe": self.recipeName,
                 "qc_name": "XRESMIN",
                 "qc_value": min_res,
-                "qc_comment": "Minimum residual in order edge fit along x-axis",
+                "qc_comment": "[px] Minimum residual in order edge fit along x-axis",
                 "qc_unit": "pixels",
                 "obs_date_utc": self.dateObs,
                 "reduction_date_utc": utcnow,
@@ -347,7 +346,7 @@ class detect_order_edges(_base_detect):
                 "soxspipe_recipe": self.recipeName,
                 "qc_name": "XRESMAX",
                 "qc_value": max_res,
-                "qc_comment": "Maximum residual in order edge fit along x-axis",
+                "qc_comment": "[px] Maximum residual in order edge fit along x-axis",
                 "qc_unit": "pixels",
                 "obs_date_utc": self.dateObs,
                 "reduction_date_utc": utcnow,
@@ -357,7 +356,7 @@ class detect_order_edges(_base_detect):
                 "soxspipe_recipe": self.recipeName,
                 "qc_name": "XRESRMS",
                 "qc_value": std_res,
-                "qc_comment": "Std-dev of residual order edge fit along x-axis",
+                "qc_comment": "[px] Std-dev of residual order edge fit along x-axis",
                 "qc_unit": "pixels",
                 "obs_date_utc": self.dateObs,
                 "reduction_date_utc": utcnow,
@@ -372,13 +371,17 @@ class detect_order_edges(_base_detect):
             self,
             orderPixelTable,
             orderPolyTable,
-            orderMetaTable):
+            orderMetaTable,
+            clippedDataUpper,
+            clippedDataLower):
         """*generate a plot of the polynomial fits and residuals*
 
         **Key Arguments:**
             - ``orderPixelTable`` -- the pixel table with residuals of fits
             - ``orderPolyTable`` -- data-frame of order-location polynomial coeff
             - ``orderMetaTable`` -- data-frame containing the limits of the fit
+            - ``clippedDataUpper`` -- the sigma-clipped data from upper edge
+            - ``clippedDataLower`` -- the sigma-clipped data from lower edge
 
         **Return:**
             - ``filePath`` -- path to the plot pdf
@@ -395,6 +398,10 @@ class detect_order_edges(_base_detect):
             orderPixelTable[f"{self.axisA}coord_lower"], orderPixelTable[f"{self.axisA}coord_upper"]))
         allAxisBCoords = np.concatenate((
             orderPixelTable[f"{self.axisB}coord"], orderPixelTable[f"{self.axisB}coord"]))
+        allAxisACoordsClipped = np.concatenate((
+            clippedDataLower[f"{self.axisA}coord_lower"], clippedDataUpper[f"{self.axisA}coord_upper"]))
+        allAxisBCoordsClipped = np.concatenate((
+            clippedDataLower[f"{self.axisB}coord"], clippedDataUpper[f"{self.axisB}coord"]))
 
         if self.axisBbin > 1:
             allAxisBCoords /= self.axisBbin
@@ -436,7 +443,8 @@ class detect_order_edges(_base_detect):
             toprow.invert_yaxis()
         toprow.set_title(
             "upper and lower order edge detections", fontsize=10)
-        toprow.scatter(allAxisBCoords, allAxisACoords, marker='x', c='red', s=0.2)
+        toprow.scatter(allAxisBCoordsClipped, allAxisACoordsClipped, marker='x', c='red', s=4, alpha=0.6, linewidths=0.5)
+        toprow.scatter(allAxisBCoords, allAxisACoords, marker='o', c='yellow', s=0.3, alpha=0.6)
         # toprow.set_yticklabels([])
         # toprow.set_xticklabels([])
         toprow.set_ylabel(f"{self.axisA}-axis", fontsize=12)
@@ -474,21 +482,38 @@ class detect_order_edges(_base_detect):
         myDict = {f"{self.axisB}": axisBlinelist}
         df = pd.DataFrame(myDict)
 
+        colors = []
         for o in uniqueOrders:
             o = int(o)
             axisBmin = orderMetaTable.loc[(orderMetaTable["order"] == o), f"{self.axisB}min"].values[0]
             axisBmax = orderMetaTable.loc[(orderMetaTable["order"] == o), f"{self.axisB}max"].values[0]
 
             df["order"] = o
-            axisAfitup = poly(df, *coeffupper)
-            axisAfitlow = poly(df, *coefflower)
+            axisAfitupStart = poly(df, *coeffupper)
+            axisAfitlowStart = poly(df, *coefflower)
 
             # xfit = np.ones(len(xfit)) * \
             #     self.flatFrame.data.shape[1] - xfit
             axisAfitup, axisBfitup = zip(
-                *[(a, b) for a, b in zip(axisAfitup, axisBlinelist) if a > 0 and a < (axisALength) - 10 and b >= axisBmin and b <= axisBmax])
+                *[(a, b) for a, b in zip(axisAfitupStart, axisBlinelist) if a > 0 and a < (axisALength) - 10])
             axisAfitlow, axisBfitlow = zip(
-                *[(a, b) for a, b in zip(axisAfitlow, axisBlinelist) if a > 0 and a < (axisALength) - 10 and b >= axisBmin and b <= axisBmax])
+                *[(a, b) for a, b in zip(axisAfitlowStart, axisBlinelist) if a > 0 and a < (axisALength) - 10])
+            if len(axisBfitlow) < len(axisBfitup):
+                half = int(len(axisAfitlowStart) / 2)
+                try:
+                    axisAfitlowExtra, axisBfitlowExtra = zip(
+                        *[(0, b) for a, b in zip(axisAfitlowStart[:half], axisBlinelist[:half]) if (a < 0 or a > (axisALength) - 10) and b in axisBfitup])
+                    axisAfitlow = axisAfitlowExtra + axisAfitlow
+                    axisBfitlow = axisBfitlowExtra + axisBfitlow
+                except:
+                    pass
+                try:
+                    axisAfitlowExtra, axisBfitlowExtra = zip(
+                        *[(0, b) for a, b in zip(axisAfitlowStart[half:], axisBlinelist[half:]) if (a < 0 or a > (axisALength) - 10) and b in axisBfitup])
+                    axisAfitlow += axisAfitlowExtra
+                    axisBfitlow += axisBfitlowExtra
+                except:
+                    pass
 
             if self.axisAbin > 1:
                 axisAfitup = np.array(axisAfitup) / self.axisAbin
@@ -498,9 +523,13 @@ class detect_order_edges(_base_detect):
                 axisBfitlow = np.array(axisBfitlow) / self.axisBbin
 
             l = midrow.plot(axisBfitlow, axisAfitlow)
+            colors.append(l[0].get_color())
             u = midrow.plot(axisBfitup, axisAfitup, c=l[0].get_color())
-            midrow.fill_between(axisBfitlow, axisAfitlow, axisAfitup, alpha=0.4, fc=l[0].get_color())
-            # midrow.text(axisBfitlow[10], axisAfitlow[10] + 5, int(o), fontsize=6, c='white', verticalalignment='bottom')
+            try:
+                midrow.fill_between(axisBfitlow, axisAfitlow, axisAfitup, alpha=0.4, fc=l[0].get_color())
+            except:
+                pass
+            midrow.text(axisBfitlow[10], axisAfitlow[10] + 5, int(o), fontsize=6, c="white", verticalalignment='bottom')
 
         # xfit = np.ones(len(xfit)) * \
         #     self.pinholeFrame.data.shape[1] - xfit
@@ -513,14 +542,22 @@ class detect_order_edges(_base_detect):
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomleft.scatter(allAxisACoords, allResiduals, alpha=0.2, s=0.05)
+        for o, c in zip(uniqueOrders, colors):
+            mask = (orderPixelTable['order'] == o)
+            orderAxisACoords = np.concatenate((orderPixelTable.loc[mask][f"{self.axisA}coord_lower"], orderPixelTable.loc[mask][f"{self.axisA}coord_upper"]))
+            orderAxisBCoords = np.concatenate((orderPixelTable.loc[mask][f"{self.axisB}coord"], orderPixelTable.loc[mask][f"{self.axisB}coord"]))
+            orderResiduals = np.concatenate((orderPixelTable.loc[mask][
+                f"{self.axisA}coord_lower_fit_res"], orderPixelTable.loc[mask][f"{self.axisA}coord_upper_fit_res"]))
+            bottomleft.scatter(orderAxisACoords, orderResiduals, alpha=0.6, s=0.2, c=c)
+            bottomleft.text(orderAxisACoords[10], orderResiduals[10], int(o), fontsize=6, c=c, verticalalignment='bottom')
+            bottomright.scatter(orderAxisBCoords, orderResiduals, alpha=0.6, s=0.2, c=c)
+            bottomright.text(orderAxisBCoords[10], orderResiduals[10], int(o), fontsize=6, c=c, verticalalignment='bottom')
         bottomleft.set_xlabel(f'{self.axisA} pixel position')
         bottomleft.set_ylabel(f'{self.axisA} residual')
         bottomleft.tick_params(axis='both', which='major', labelsize=9)
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomright.scatter(allAxisBCoords, allResiduals, alpha=0.2, s=0.2)
         bottomright.set_xlabel(f'{self.axisB} pixel position')
         bottomright.tick_params(axis='both', which='major', labelsize=9)
         # bottomright.set_ylabel(f'{self.axisA} residual')
@@ -532,8 +569,6 @@ class detect_order_edges(_base_detect):
         subtitle = f"mean res: {mean_res:2.2f} pix, res stdev: {std_res:2.2f}"
         fig.suptitle(f"detection of order-edge locations - flat-frame\n{subtitle}", fontsize=12)
 
-        # plt.show()
-
         filename = filenamer(
             log=self.log,
             frame=self.flatFrame,
@@ -544,6 +579,7 @@ class detect_order_edges(_base_detect):
         outDir = self.settings["intermediate-data-root"].replace("~", home) + "/qc/pdf"
         filePath = f"{outDir}/{filename}"
         plt.tight_layout()
+        # plt.show()
         plt.savefig(filePath, dpi=720)
 
         self.log.debug('completed the ``plot_results`` method')
@@ -601,10 +637,8 @@ class detect_order_edges(_base_detect):
             medSlide[int(len(medSlide) / 2 - 8):int(len(medSlide) / 2 + 8)])
         minvalue = np.min(medSlide)
 
-        orderData["minThreshold"] = minvalue + \
-            (maxvalue - minvalue) * minThresholdPercenage
-        orderData["maxThreshold"] = minvalue + \
-            (maxvalue - minvalue) * maxThresholdPercenage
+        orderData["minThreshold"] = minvalue + (maxvalue - minvalue) * minThresholdPercenage
+        orderData["maxThreshold"] = minvalue + (maxvalue - minvalue) * maxThresholdPercenage
 
         # SANITY CHECK PLOT OF CROSS-SECTION
         if 1 == 0:
