@@ -11,12 +11,7 @@
 """
 ################# GLOBAL IMPORTS ####################
 from soxspipe.commonutils import keyword_lookup
-from astropy import units as u
-import ccdproc
-from astropy.nddata import CCDData
-import numpy as np
 from ._base_recipe_ import _base_recipe_
-from soxspipe.commonutils import set_of_files
 from fundamentals import tools
 from builtins import object
 import sys
@@ -34,6 +29,8 @@ class soxs_spatial_solution(_base_recipe_):
         - ``settings`` -- the settings dictionary
         - ``inputFrames`` -- input fits frames. Can be a directory, a set-of-files (SOF) file or a list of fits frame paths
         - ``verbose`` -- verbose. True or False. Default *False*
+        - ``overwrite`` -- overwrite the prodcut file if it already exists. Default *False*
+        - ``create2DMap`` -- create the 2D image map of wavelength, slit-position and order from disp solution.
 
     See `produce_product` method for usage.
 
@@ -41,6 +38,7 @@ class soxs_spatial_solution(_base_recipe_):
     .. todo::
 
         - add a tutorial about ``soxs_spatial_solution`` to documentation
+
     ```
     """
     # Initialisation
@@ -50,28 +48,33 @@ class soxs_spatial_solution(_base_recipe_):
             log,
             settings=False,
             inputFrames=[],
-            verbose=False
+            verbose=False,
+            overwrite=False,
+            create2DMap=True
 
     ):
         # INHERIT INITIALISATION FROM  _base_recipe_
         super(soxs_spatial_solution, self).__init__(
-            log=log, settings=settings)
+            log=log, settings=settings, inputFrames=inputFrames, overwrite=overwrite, recipeName="soxs-spatial-solution")
         self.log = log
         log.debug("instansiating a new 'soxs_spatial_solution' object")
         self.settings = settings
         self.inputFrames = inputFrames
         self.verbose = verbose
-        self.recipeName = "soxs-spatial-solution"
         self.recipeSettings = settings[self.recipeName]
+        self.create2DMap = create2DMap
+
         # xt-self-arg-tmpx
 
         # INITIAL ACTIONS
         # CONVERT INPUT FILES TO A CCDPROC IMAGE COLLECTION (inputFrames >
         # imagefilecollection)
+        from soxspipe.commonutils.set_of_files import set_of_files
         sof = set_of_files(
             log=self.log,
             settings=self.settings,
-            inputFrames=self.inputFrames
+            inputFrames=self.inputFrames,
+            ext=self.settings['data-extension']
         )
         self.inputFrames, self.supplementaryInput = sof.get()
 
@@ -83,7 +86,7 @@ class soxs_spatial_solution(_base_recipe_):
         print("# VERIFYING INPUT FRAMES - ALL GOOD")
 
         # SORT IMAGE COLLECTION
-        self.inputFrames.sort(['mjd-obs'])
+        self.inputFrames.sort(['MJD-OBS'])
         if self.verbose:
             print("# RAW INPUT FRAMES - SUMMARY")
             print(self.inputFrames.summary, "\n")
@@ -171,6 +174,9 @@ class soxs_spatial_solution(_base_recipe_):
         """
         self.log.debug('starting the ``produce_product`` method')
 
+        from astropy.nddata import CCDData
+        from astropy import units as u
+
         arm = self.arm
         kw = self.kw
         dp = self.detectorParams
@@ -183,13 +189,13 @@ class soxs_spatial_solution(_base_recipe_):
         multi_pinhole_image = False
         order_table = False
 
-        add_filters = {kw("DPR_CATG"): 'MASTER_BIAS_' + arm}
+        add_filters = {kw("PRO_CATG"): 'MASTER_BIAS_' + arm}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
             master_bias = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                        hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
 
         # UVB/VIS DARK
-        add_filters = {kw("DPR_CATG"): 'MASTER_DARK_' + arm}
+        add_filters = {kw("PRO_CATG"): 'MASTER_DARK_' + arm}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
             dark = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                 hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
@@ -202,7 +208,7 @@ class soxs_spatial_solution(_base_recipe_):
                                 hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
 
         # UVB/VIS/NIR FLAT
-        add_filters = {kw("DPR_CATG"): 'MASTER_LAMP-FLAT_' + arm}
+        add_filters = {kw("PRO_CATG"): 'MASTER_FLAT_' + arm}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
             master_flat = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                        hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
@@ -217,7 +223,7 @@ class soxs_spatial_solution(_base_recipe_):
         self.dateObs = multi_pinhole_image.header[kw("DATE_OBS")]
 
         # FIND THE ORDER TABLE
-        filterDict = {kw("PRO_CATG").lower(): f"ORDER_TAB_{arm}"}
+        filterDict = {kw("PRO_CATG"): f"ORDER_TAB_{arm}"}
         order_table = self.inputFrames.filter(**filterDict).files_filtered(include_path=True)[0]
 
         add_filters = {kw("PRO_CATG"): f"DISP_TAB_{arm}".upper()}
@@ -230,7 +236,7 @@ class soxs_spatial_solution(_base_recipe_):
         if self.settings["save-intermediate-products"]:
             fileDir = self.intermediateRootPath
             filepath = self._write(
-                self.multiPinholeFrame, fileDir, filename=False, overwrite=True)
+                self.multiPinholeFrame, fileDir, filename=False, overwrite=True, product=False)
             print(f"\nCalibrated multi pinhole frame frame saved to {filepath}\n")
 
         # GENERATE AN UPDATED DISPERSION MAP
@@ -242,7 +248,9 @@ class soxs_spatial_solution(_base_recipe_):
             firstGuessMap=disp_map_table,
             orderTable=order_table,
             qcTable=self.qc,
-            productsTable=self.products
+            productsTable=self.products,
+            sofName=self.sofName,
+            create2DMap=self.create2DMap
         ).get()
 
         from datetime import datetime
@@ -265,17 +273,18 @@ class soxs_spatial_solution(_base_recipe_):
             "file_path": productPath
         }, ignore_index=True)
 
-        filename = os.path.basename(mapImagePath)
-        self.products = self.products.append({
-            "soxspipe_recipe": self.recipeName,
-            "product_label": "2D_MAP",
-            "file_name": filename,
-            "file_type": "FITS",
-            "obs_date_utc": self.dateObs,
-            "reduction_date_utc": utcnow,
-            "product_desc": f"{self.arm} 2D detector map of wavelength, slit position and order",
-            "file_path": productPath
-        }, ignore_index=True)
+        if mapImagePath:
+            filename = os.path.basename(mapImagePath)
+            self.products = self.products.append({
+                "soxspipe_recipe": self.recipeName,
+                "product_label": "2D_MAP",
+                "file_name": filename,
+                "file_type": "FITS",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow,
+                "product_desc": f"{self.arm} 2D detector map of wavelength, slit position and order",
+                "file_path": productPath
+            }, ignore_index=True)
 
         self.report_output()
 
